package OutputResults;
use base "Exporter";
our @EXPORT = qw(outputResults);

use List::Util qw( min max );

#use strict;
#use warnings;

#Outputs the results in the format expected for SemEval
#   Input - \@words1, \@words2 = the two input sentences
#           - \@chunks1, \@chunks2 = the chunk mapping for the two input sentences
#           - \@alignments, \alignmentmentReasons, \@alignmentScores = the alignments for each word in the two sentences, the reasons and the scores
#           - $sentenceID = the sentenceID that corresponds to the gold standard output
#           - $resultsFileName = the fileName to which the results will be output
#           - $swapped = boolean indicating whether or not the order of the two sentences was swapped
#   Returns - nothing but outputs the provided output filename
sub outputResults
{
    #grab input variables
    my @words1 = @{ shift @_ };
    my @words2 = @{ shift @_ };
    my @chunks1 = @{ shift @_ };
    my @chunks2 = @{ shift @_ }; 
    my @alignments = @{ shift @_ };
    my @alignmentReasons = @{ shift @_ };
    my @alignmentScores = @{ shift @_ };
    my $sentenceID = shift;
    my $resultsFileName = shift;
    my $swapped = shift;

    #swap the word orders if swapped
    if ($swapped) {
	my @temp = @words1;
	@words1 = @words2;
	@words2 = @temp;
    }


    #open the output file
    open (OUT, ">>".$resultsFileName);

    #print the original sentences
    print OUT "<sentence id=\"$sentenceID\" status=\"\">\n";
    print OUT "\/\/ ".join(" ", @words1)."\n";
    print OUT "\/\/ ".join(" ", @words2)."\n";
    print "<sentence id=\"$sentenceID\" status=\"\">\n";
    print "\/\/ ".join(" ", @words1)."\n";
    print "\/\/ ".join(" ", @words2)."\n";


    #print the original words (tokens) of sentence 1
    print OUT "<source>\n";
    print "<source>\n";
    my $count = 1;
    foreach my $word(@words1) {
	print OUT "$count $word :\n";
	print "$count $word :\n";
	$count++;
    }
    print OUT "<\/source>\n";
    print "<\/source>\n";

    #print the original words (tokens) of sentence 2
    print OUT "<\/translation>\n";
    print "<\/translation>\n";
    $count = 1;
    foreach my $word(@words2) {
	print OUT "$count $word :\n";
	print "$count $word :\n";
	$count++;
    }
    print OUT "<\/translation>\n";
    print "<\/translation>\n";

    if ($swapped == 0) {
	&outputAlignments(\@words1, \@words2, 
			  \@chunks1, \@chunks2, 
			  \@alignments, 
			  \@alignmentReasons, \@alignmentScores);
    }
    else {
	&outputSwappedAlignments(\@words1, \@words2, 
				 \@chunks1, \@chunks2, 
				 \@alignments, 
				 \@alignmentReasons, \@alignmentScores);
    }
    
    print OUT "<\/alignment>\n";
    print OUT "<\/sentence>\n";
    print OUT "\n\n";
    print "<\/alignment>\n";
    print "<\/sentence>\n"; 
    print "\n\n";
}


sub outputAlignments
{
    #grab the input
    my @words1 = @{ shift @_ };
    my @words2 = @{ shift @_ };
    my @chunks1 = @{ shift @_ };
    my @chunks2 = @{ shift @_ };
    my @alignments = @{ shift @_ };
    my @alignmentReasons = @{ shift @_ };
    my @alignmentScores = @{ shift @_ };

    print "<alignment>\n";
    print OUT "<alignment>\n";

    #create a datastructure to look for no alignments in the second chunk
    my @chunk2Prints = ();
    foreach my $i(@chunks2) {
	push (@chunk2Prints, 0);
    }

    #print all chunks1 alignments
    my $numChunks1 = (max @chunks1) + 1;
    for (my $chunkNumber1 = 0; $chunkNumber1 < $numChunks1; $chunkNumber1++) {

	#print left hand indeces
	my $printed = 0;
	my @alignedChunks = ();
	for (my $i = 0; $i < scalar @chunks1; $i++) {
	    if ($chunks1[$i] == $chunkNumber1) {
		$printed = 1;
		my $index = $i + 1;
		print "$index ";
		print OUT "$index ";
		push (@alignedChunks, $alignments[$i]);
	    }
	}

	#check if something was printed, if so continue to print the alignments
	if ($printed > 0) {

	    print "<==> ";
	    print OUT "<==> ";

	    #print right hand indeces
	    $printed = 0;
	    for (my $i = 0; $i < scalar @chunks2; $i++) {
		if (($chunks2[$i]+1) == $alignments[$chunkNumber1]) {
		    $printed = 1;
		    my $index = $i+1;
		    print "$index ";
		    print OUT "$index ";
		    $chunk2Prints[$i]++;
		}
	    }
	    if ($printed == 0) {
		print "0 ";  #corresponds to no alignment
		print OUT "0 ";
	    }

	    #print alignment score and reason
	    print " \/\/ ".$alignmentReasons[$chunkNumber1+1];
	    print " \/\/ ".$alignmentScores[$chunkNumber1+1];
	    print " \/\/ ";
	    print OUT " \/\/ ".$alignmentReasons[$chunkNumber1+1];
	    print OUT " \/\/ ".$alignmentScores[$chunkNumber1+1];
	    print OUT " \/\/ ";

	    #print left hand words
	    for (my $i = 0; $i < scalar @chunks1; $i++) {
		if ($chunks1[$i] == $chunkNumber1) {
		    print "$words1[$i] ";
		    print OUT "$words1[$i] ";
		}
	    }
	    print "<==> ";
	    print OUT "<==> ";

	    #print right hand words
	    $printed = 0;
	    for (my $i = 0; $i < scalar @chunks2; $i++) {
		    if (($chunks2[$i]+1) == $alignments[$chunkNumber1]) {
			$printed = 1;
			print "$words2[$i] ";
			print OUT "$words2[$i] ";
		    }
	    }
	    if ($printed == 0) {
		print "-not aligned- ";  #corresponds to no alignment
		print OUT "-not aligned- ";
	    }

	    print "\n";
	    print OUT "\n";
	}
    }

    #print any no alignments from chunks2
    for (my $i = 0; $i < scalar @chunk2Prints; $i++) {
	if ($chunk2Prints[$i] == 0) {
	    #this chunk was not printed, so print it now
	    my $chunkToPrint = $chunks2[$i];
	    print "0 <==> ";
	    print OUT "0 <==> ";

	    #print the indeces
	    for (my $j = 0; $j < scalar @words2; $j++) {
		if ($chunks2[$j] == $chunkToPrint) {
		    my $index = $j+1;
		    print "$index ";
		    print OUT "$index ";
		    $chunk2Prints[$j]++;
		}
	    }

	    print "\/\/ NOALI \/\/ 0 \/\/ -not aligned- <==> ";
	    print OUT "\/\/ NOALI \/\/ 0 \/\/ -not aligned- <==> ";

	    #print the words
	    for (my $j = 0; $j < scalar @words2; $j++) {
		if ($chunks2[$j] == $chunkToPrint) {
		    print "$words2[$j] ";
		    print OUT "$words2[$j] ";
		}
	    }
	    print "\n";
	    print OUT "\n";
	}
    }
}


sub outputSwappedAlignments
{
    #grab the input
    my @words1 = @{ shift @_};
    my @words2 = @{ shift @_ };
    my @chunks1 = @{ shift @_ };
    my @chunks2 = @{ shift @_ };
    my @alignments = @{ shift @_ };
    my @alignmentReasons = @{ shift @_ };
    my @alignmentScores = @{ shift @_ };

    print "<alignment>\n";
    print OUT "<alignment>\n";

    #create a datastructure to look for no alignments in the second chunk
    my @chunk2Prints = ();
    foreach my $i(@chunks2) {
	push (@chunk2Prints, 0);
    }

    #print left hand indeces
    my $numChunks1 = (max @chunks1) + 1;
    for (my $chunk1Number = 0; $chunk1Number < $numChunks1; $chunk1Number++) {
	#print left hand indeces
	my $printed = 0;
	for (my $i = 0; $i < scalar @chunks2; $i++) {
	    if (($chunks2[$i]+1) == $alignments[$chunk1Number]) {
		$printed = 1;
		my $index = $i+1;
		print "$index ";
		print OUT "$index ";
		$chunk2Prints[$i]++;
	    }
	}
	if ($printed == 0) {
	    print "0 "; #corresponds to a no alignment
	    print OUT "0 ";
	}

	print "<==> ";
	print OUT "<==> ";

	#print right hand indeces
	for (my $i = 0; $i < scalar @chunks1; $i++) {
	    if ($chunks1[$i] == $chunk1Number) {
		my $index = $i+1;
		print "$index ";
		print OUT "$index ";
	    }
	}

	#print alignment score and reason
	print " \/\/ ".$alignmentReasons[$chunk1Number+1];
	print " \/\/ ".$alignmentScores[$chunk1Number+1];
	print " \/\/ ";
	print OUT " \/\/ ".$alignmentReasons[$chunk1Number+1];
	print OUT " \/\/ ".$alignmentScores[$chunk1Number+1];
	print OUT " \/\/ ";

	#print right hand words
	$printed = 0;
	for (my $i = 0; $i < scalar @chunks2; $i++) {
	    if (($chunks2[$i]+1) == $alignments[$chunk1Number]) {
		$printed = 1;
		print "$words1[$i] ";
		print OUT "$words1[$i] ";
	    }
	}
	if ($printed == 0){
	    print "-not aligned- ";
	    print OUT "-not aligned- ";
	}

	print "<==> ";
	print OUT "<==> ";

	#print left hand words
	for (my $i = 0; $i < scalar @chunks1; $i++) {
	    if ($chunks1[$i] == $chunk1Number) {
		print "$words2[$i] ";
		print OUT "$words2[$i] ";
	    }
	}
	print "\n";
	print OUT "\n";
    }

    #print any no alignments from chunks2
    for (my $i = 0; $i < scalar @chunk2Prints; $i++) {
	if ($chunk2Prints[$i] == 0) {
	    #this chunk was not printed, so print it now
	    my $chunkToPrint = $chunks2[$i];

	    #print the indeces
	    for (my $j = 0; $j < scalar @chunks2; $j++) {
		if ($chunks2[$j] == $chunkToPrint) {
		    my $index = $j+1;
		    print "$index ";
		    print OUT "$index ";
		    $chunk2Prints[$j]++;
		}
	    }

	    print "<==> 0 \/\/ NOALI \/\/ 0 \/\/ ";
	    print OUT "<==> 0 \/\/ NOALI \/\/ 0 \/\/ ";

	    #print the words
	    for (my $j = 0; $j < scalar @words1; $j++) {
		if ($chunks2[$j] == $chunkToPrint) {
		    print "$words1[$j] ";
		    print OUT "$words1[$j] ";
		}
	    }
	    print " <==> -not aligned-";
	    print "\n";
	    print OUT " <==> -not aligned-";
	    print OUT "\n";
	}
    }
}



1;
=comment
@author = Sam Henry

@Descriptions - Outputs the results of chunk alignment to a file that is readable by the official evaluation software

=cut 
