#!/usr/bin/perl
package ReadGoldOutput;  #the package name
use base "Exporter";
@EXPORT = qw(readGold);  #telling what we should export

# puts gold output into an array of hash tables

use CreateChunks;
#use strict;
#use warnings;

####################################################################
# ReadGoldOutput.pm
# Allison Sands, edited by Sam Henry
# Input: Reads both chunked files provided by SemEval
# as well as the output from Stage one of project (as a file 
# Returns as output an array (@sentencePair) of hashes of arrays and scalars for each pair of sentences
#		\@words1 \@words2 -- Arrays of word from "matching" sentences of each file
#		\@chunks1 \@chunks2 -- Arrays of chunks of each sentence
#		\@alignmentReasons -- Array of reasons for sentence "matching" (or not)
#					including OPPO, EQUI, SPE1, SPE2, SIMI, NOALI
#		\@alignmentScores -- Array of integers from 0 - 5 that correspond
#					with alignmentReasons above
#		\@alignments -- Array of length of longer sentence, each element
#				contains number of chunk corresponding from shorter sentence
#		$swapped -- set to true if sentence1 is longer than sentence2
#		$fact -- set to true if statement of fact
#		$pol -- polarity indication
# Makes a call to CreateChunks.pm for creation of @chunks1 and @chunks2
##########################################################################

####################################################################
sub readGold  #returns array of hash tables of gold output
{ 
    #grab input
    my $goldOutputFile = shift;

    #initialize data structures
    my @sentencePair; #array of hashes
    my $index = 0;

    #open files
    open(my $src, $goldOutputFile) or die "gold file not opened: $goldOutputFile\n";

    #grab all sentences in the files
    while(!eof($src)) { 
	    
#-------------read in the chunked sentences

	#sentence 1 creation
        <$src>;  #skip the <sentence> line
	my $thisSentence1 = <$src>;		#// A cat
	#chomp $thisSentence1;
	$thisSentence1 =~ s/\/\/\s//;
	my @sentence1 = split / /, $thisSentence1;   #creation of @sentence1

	#sentence 2 creation
	my $thisSentence2 = <$src>;		#// A black and white cat
	#chomp $thisSentence2;
	$thisSentence2 =~ s/\/\/\s//;
	my @sentence2 = split / /, $thisSentence2;  #creation of @sentence2
	
       

#--------- Read in the Words
	 <$src>; #skip the <source> label

	#read in words1
	my @words1 = ();
	for (my $i = 0; $i < scalar @sentence1; $i++) {  #1 A : 2 cat : standing :			   
	    $words1[$i] = <$src>;
	    chomp $words1[$i];
	    $words1[$i] =~ s/\d+\s//; #dumps leading digits
	    $words1[$i] =~ s/\s\://;  #dumps trailing colon
	}
	<$src>; #skip the </source> label
	<$src>; #skip the </translation> label

	#read in words2
	my @words2 = ();
	for (my $i = 0; $i < scalar @sentence2; $i++) {  # A : 2 black : 3 and : 4 : white :                 
	    $words2[$i] = <$src>;
	    chomp $words2[$i];
	    $words2[$i] =~ s/\d+\s//; #dumps leading digits
	    $words2[$i] =~ s/\s\://;  #dumps trailing colon
	}
	<$src>; #skip the </translation> label


#------------ swap the sentences if needed

	#words1 should always be >= length of sentence 2 (for indexing)
	my $swapped = scalar @words1 < scalar @words2;
	if ($swapped) {
	    my @temp = @words1;
	    @words1 = @words2;
	    @words2 = @temp;
	}


#-------- Read in the alignments

	#create the data structures
	my @alignmentReasons; # EQUI, OPPO, etc.
	my @alignmentScores; # range from 1 to 5
	my @alignments = ();
	my $thisSizeAlign = scalar @alignments;
        my @chunks1 = ();
        my @chunks2 = ();
	
	#add no alignment to the reasons and scores
	push @alignmentReasons, 'NOALI';
	push @alignmentScores, 0;

	#process all the alignments lines
	my $alignmentNumber = 0;  
	<$src>; #skip the <alignment> label
	my $done = 0;
	my $incrementAlignmentNumber = 0;
	while (!$done)
	{
	    #read in the line
	    my $alignmentText = <$src>;
	    chomp $alignmentText;
	    
	    #if the line read in is an alignment line
	    if ($alignmentText =~ m/^\d+\s/) 
	    {   
		#read in the word indeces (1 2 3 <==> 1 2)
		my @wordNumbers1 = ();
		my @wordNumbers2 = ();
		if ($alignmentText =~ m/([\d+\s]+)\s<=/) {
		    @wordNumbers1 = split(' ', $1);
		}
		if ($alignmentText =~ m/=>\s([\d+\s]+)/) { 
		    @wordNumbers2 = split(' ', $1);
		}

		#subtract one from each word number so that they are indeces
		#   in the file word numbers start at 1 rather than 0
		for (my $i = 0; $i < scalar @wordNumbers1; $i++) {
		    $wordNumbers1[$i]--;
		}
		for (my $i = 0; $i < scalar @wordNumbers2; $i++) {
		    $wordNumbers2[$i]--;
		}

		#swap if needed
		if ($swapped) {
		    my @temp = @wordNumbers1;
		    @wordNumbers1 = @wordNumbers2;
		    @wordNumbers2 = @temp;
		}


		#figure out the chunks from the read in word numbers
		if ($wordNumbers1[0] >= 0) {
		    $incrementAlignmentNumber = 1;  #only increment the alignment chunk1 gets aligned
		    for (my $i = 0; $i < scalar @wordNumbers1; $i++) {
			    $chunks1[$wordNumbers1[$i]] = $alignmentNumber;
		    }
		}
		if ($wordNumbers2[0] >= 0) {
		    for (my $i = 0; $i < scalar @wordNumbers2; $i++) {
			#check that it is defined because multiple alignments with the same chunk are possible
			if (!(defined $chunks2[$wordNumbers2[$i]])) {
			    $chunks2[$wordNumbers2[$i]] = $alignmentNumber;
			}
		    }
		}

		#set the alignment
		if ($wordNumbers1[0] >= 0) {
		    if ($wordNumbers2[0] >= 0) {
			#alignment exists
			$alignments[$alignmentNumber] = $chunks2[$wordNumbers2[0]] + 1;
		    }
		    else {
			#no alignment
			$alignments[$alignmentNumber] = 0;
		    }
		}

		#grab the reason and scores
		if ($alignmentText =~ m/\/\/\s((NOALI|EQUI|OPPO|SPE1|SPE2|SIMI|REL))/) { 
		    $alignmentReasons[$alignmentNumber + 1] = $1;
		}
		if ($alignmentText =~ m/\/\/\s([0-5]|NIL)/) { 
		    $alignmentScores[$alignmentNumber + 1] = $1; 
		    if ($alignmentScores[$alignmentNumber + 1] eq "NIL") {
			$alignmentScores[$alignmentNumber + 1] = 0;
		    }
		}
	    }
	    else {  #all numbers have been processed, quit
		$done = 1; 
		last;
	    }
	    
	    #increment the alignmentNumber
	    if ($incrementAlignmentNumber > 0) {
		$alignmentNumber++;
	    }
	    $incrementAlignmentNumber = 0;

	} #while begins with digit

	#save this sentence pair
	my $fact = 0; #TODO read these in 
	my $pol = 0;
	my %newHash = ();
	$newHash{swapped} = $swapped;  
	$newHash{chunks1} = \@chunks1;
	$newHash{chunks2} = \@chunks2;
	$newHash{words2} = \@words2;
	$newHash{words1} = \@words1;
	$newHash{alignments} = \@alignments;
	$newHash{alignmentReasons} = \@alignmentReasons;
	$newHash{alignmentScores} = \@alignmentScores;
	$newHash{fact} = $fact;
	$newHash{polarity} = $pol;
	#$sentencePair[$index] = \%newHash;
	push @sentencePair, \%newHash;

	#skip lines
	#my $alignmentEnd = $alignmentArrayTemp;
	<$src>; #skip the </alignment> label
	<$src>; #skip the </sentence> label
	<$src>; #skip the blank line
        #<$src>; #skip the sencond black line

	#increment the index
	$index++;
     }

    #return all the sentence pairs read in
    return \@sentencePair;
}

1;
